module Slds
  class SldsController < ActionController::Base

    rescue_from StandardError, with: :render_error
    layout false

    def show
      if slds_response.code == '200' # set our response headers
        expires_in Slds.max_age, 's-maxage' => Slds.s_maxage, public: true
        response.headers['Surrogate-Key'] = 'DFC-SLDS-ASSETS' # allow these to be expired from fastly as a group
      elsif slds_response.code == '404'
        slds_response.body = 'not found'
      end

      render(
        body: slds_response.body,
        content_type: slds_response.content_type,
        status: slds_response.code
      )
    end

    private

    #
    # http://opensourceconnections.com/blog/2008/04/24/adding-timeout-to-nethttp-get_response/
    #
    # Net::HTTP.get and Net::HTTP.get_response
    # The difference between these two methods is that
    # get_response is a class method that returns an Net::HTTPResponse object
    # while get is an instance method that returns a string.
    # And get_response doesn’t support passing in timeouts of any kind!
    # By wrapping my call to http.get in an http.start{} block,
    # I get the timeout ability of http.get
    # with the return of an HTTPResponse that Net::HTTP.get_response normally provides.
    def slds_response
      return @slds_response if @slds_response
      http = Net::HTTP.new(item_uri.host, item_uri.port)
      http.use_ssl = true
      http.read_timeout = 2
      http.open_timeout = 2
      @slds_response ||= http.start do |http|
        http.get(item_uri.path)
      end
    end

    # Map the given URL path to the absolute url to the Slds asset uri (server).
    #
    # For example: Slds.assets_path('/icons/utility-sprite/svg/symbols.svg#right') =>
    #   "/slds/2.2.1/assets/icons/utility-sprite/svg/symbols.svg#right"
    #  which maps to this path in slds itself:
    #     => 'https://unpkg.com/@salesforce-ux/design-system@2.2.1/assets/icons/utility-sprite/svg/symbols.svg#right"
    def item_uri
      path = params[:id] # eg "/icons/utility-sprite/svg/symbols.svg"
      version = params[:version] # eg "2.2.1"
      url = Slds.assets_url(path, version) # eg "https://unpkg.com/@salesforce-ux/design-system@#{version}/assets/icons/utility-sprite/svg/symbols.svg"
      logger.debug("Slds::SldsController.item_uri: path #{path} proxied to #{url} using version #{version}") if Slds.debug

      URI(url)
    end

    def render_error(e)
      logger.error("Slds::SldsController.render_error: exception: #{e}")
      logger.error(e.backtrace.first(2))

      render(status: 500, body: 'error')
    end
  end
end
