require 'slds/engine'
require 'slds/helper'

module Slds
  extend self
  attr_accessor :version
  attr_accessor :url
  attr_accessor :logger
  attr_accessor :max_age
  attr_accessor :s_maxage
  attr_accessor :debug # for logging in the controller for each request

  self.version = '2.2.1'
  self.url = "https://unpkg.com/@salesforce-ux/design-system@VERSION/assets/"
  self.max_age = 86400
  self.s_maxage = 86400
  self.debug = false

  # Set the version and the url (dont mess with the url unless you know what you are doing)
  # Usage: Slds.config { |slds| slds.version = '2.2.1'; slds.debug = true }
  def config(options = {})
    if options.present?
      options.each { |key, value| send("#{key}=", value) }
    else
      yield self
    end

    validate_version(version)
  end

  # creates a absolute url that uses the url (unpkg.com) by prepending the url parameter.
  # Note that /assets/ should not be part of your path.
  # Slds.assets_url('styles/salesforce-lightning-design-system.min.css')
  #   => 'https://unpkg.com/@salesforce-ux/design-system@2.2.1/assets/styles/salesforce-lightning-design-system.min.css'
  def assets_url(path, v = version)
    validate_version(v)
    url.gsub('VERSION', v) + trim_path(path)
  end

  # creates a relative path for the slds asset that uses the version and starts with /slds
  # For example: Slds.assets_path('/icons/utility-sprite/svg/symbols.svg')
  #   creates this url => "/slds/2.2.1/icons/utility-sprite/svg/symbols.svg"
  #
  # This path can be put in your app and the Slds controller (of this gem) will handle the path
  # (because it starts with /slds) and make a reverse proxy Net::http call to the url.
  #
  # The above url makes a http call to:
  #   => 'https://unpkg.com/@salesforce-ux/design-system@2.2.1/assets/icons/utility-sprite/svg/symbols.svg"
  def assets_path(path, v = version)
    validate_version(v)
    "/slds/#{v}/#{trim_path(path)}"
  end

  private

  def validate_version(version)
    raise ArgumentError.new("Slds: version #{version} not of the form 2.2.2 or 2.1 or 2 or 2.3.4.5 etc.") unless version =~ /^[0-9\.]+$/
  end

  # remove any leading / in the path (aka lchomp)
  def trim_path(path)
    path[0] == '/' ? trim_path(path[1..-1]) : path
  end
end
